@@ -10,13 +10,15 @@ from .popup_handler import PopupHandler
 from .table_handler import TableHandler
 from .textbox_handler import TextboxHandler
 from .ui_context import UIContext
-from .misc import Misc
+from .base_context import BaseContext
+from playwright.sync_api import sync_playwright
 
 
 class PlayerLibrary(DynamicCore):
+    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
 
     def __init__(self):
-        libraries = [APIContext(), UIContext(), ButtonHandler(), CheckboxHandler(), DatePickerHandler(),
-                     DropdownHandler(), IframeHandler(), PageHandler(), PopupHandler(), TableHandler(),
-                     TextboxHandler(), Misc()]
+        libraries = [BaseContext(), APIContext(), UIContext(), ButtonHandler(), CheckboxHandler(),
+                     DatePickerHandler(), DropdownHandler(), IframeHandler(), PageHandler(), 
+                     PopupHandler(), TableHandler(), TextboxHandler()]
         DynamicCore.__init__(self, libraries)
