@@ -1,7 +1,7 @@
 from playwright.sync_api import expect
 from robotlibcore import keyword
 from .ui_context import UIContext
-from .base_context import SMALL_TIMEOUT
+from .base_context import BaseContext
 
 
 class ButtonHandler(UIContext):
@@ -9,12 +9,12 @@ class ButtonHandler(UIContext):
     @keyword('button should be enabled')
     def button_should_be_enabled(self, locator):
         element = self.get_element(locator)
-        expect(element).to_be_enabled(timeout=SMALL_TIMEOUT)
+        expect(element).to_be_enabled(timeout=BaseContext.SMALL_TIMEOUT)
 
     @keyword('button should be disabled')
     def button_should_be_disabled(self, locator):
         element = self.get_element(locator)
-        expect(element).to_be_disabled(timeout=SMALL_TIMEOUT)
+        expect(element).to_be_disabled(timeout=BaseContext.SMALL_TIMEOUT)
 
     @keyword('button should be correct')
     def button_should_be_correct(self, locator, state='enabled'):
@@ -22,6 +22,6 @@ class ButtonHandler(UIContext):
         self.button_should_be_enabled(element) if state == 'enabled' else self.button_should_be_disabled(element)
 
     @keyword('click button')
-    def click_button(self, locator):
+    def click_button(self, locator, force=False):
         element = self.get_element(locator)
-        element.click(force=True)
+        element.click(force=force)
